/*
Package fastjson provides fast JSON parsing.

Arbitrary JSON may be parsed by fastjson without the need for creating structs
or for generating go code. Just parse JSON and get the required fields with
Get* functions.

*/
package fastjson
